/*
 *
 * Copyright (c) 2004  DMC Co., Ltd.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE X CONSORTIUM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF
 * OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 * 
 * Except as contained in this notice, the name of the DMC Co., Ltd shall not 
 * be used in advertising or otherwise to promote the sale, use or other dealings
 * in this Software without prior written authorization from DMC Co., Ltd.
 */
/*
 * EUC
 * tab=4
 */
#include <X11/Xlib.h>
#include <X11/extensions/XI.h>
#include <X11/extensions/XInput.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
//#include <curses.h>
#include "tb.h"
#include "../driver/cmd.h"

/* define after x-org v1.4.0 */
#if 1
#define	XVERS14		1
#endif
/* define after x-org v1.4.0 */

#define	GDISP	gDisp	/* NG=gDisp2[snum] */

#if 1
#define	PRINT(a,b,c,d,e)	printf(a,b,c,d,e)
#else
#define	PRINT(a,b,c,d,e)
#endif

static XDeviceInfo *info[MAX_SCREEN];
static XDevice *dev[MAX_SCREEN];
static XDeviceControl *dctl[MAX_SCREEN];

static int do_set_calib(int snum,u_short dat);
static XDeviceInfo *find_device_info(Display *display,char *name, Bool only_extended);

#if USE_PIPE_COMM
static int pipe_open(char *name);
static int send_dmcpkt(int snum,int cmd,char *dat,int size);
static int recv_dmcpkt(int snum,int cmd);
static int send_dmcinpkt(int snum,int cmd,char *dat,int size);
static int recv_dmcinpkt(int snum,int cmd,void *dat,int size);

static int pinfd[MAX_SCREEN];
static int poutfd[MAX_SCREEN];
char rxname[] = DMC_PIPE_RX_FILE;
char txname[] = DMC_PIPE_TX_FILE;
#endif

int gScreenCount = 0;
char *drvname[] = { "DMC" , "DMC1" };

static int do_open_xinput_driver(int snum);
static int do_set_raw_mode(int snum,int on);
static int do_is_selector_on(int snum);
static int do_select_button(int snum,int bttn);	/* 0=left/1=middle/2=right */
static int do_reset_tb(int snum);
static int do_set_samp_rate(int snum,int s_rate);
static int do_get_samp_rate(int snum);
static int do_set_beep(int snum,int pen_down,int on);
static int do_get_beep(int snum);
static int do_set_offdelay(int snum,int msec);
static int do_get_offdelay(int snum);
static int do_save_exec(int snum);
static int do_get_calib_point_num(int snum);

int open_xinput_driver(char *driver_name)
{
int snum,cnt;
	for( cnt=snum=0 ; snum < MAX_SCREEN ; snum++ )
	{
		if( do_open_xinput_driver(snum) == 0 )
			cnt++;
	}
	gScreenCount = cnt;
	return( cnt ? 0 : -1 );
}
static int do_open_xinput_driver(int snum)
{
int ctype,res;
char s[512];
#if USE_PIPE_COMM
	if( snum == 0 )
		sprintf(s,"%s",txname);
	else
		sprintf(s,"%s%d",txname,snum);
	if( (poutfd[snum]=pipe_open(s)) == -1 )
	{
		PRINT("error : %s out-pipe open\n",drvname[snum],0,0,0);
		return(-1);
	}
	PRINT("device [%s] out-pipe open [%s] %d\n",drvname[snum],s,poutfd[snum],0);
	if( snum == 0 )
		sprintf(s,"%s",rxname);
	else
		sprintf(s,"%s%d",rxname,snum);
	if( (pinfd[snum]=pipe_open(s)) == -1 )
	{
		PRINT("error : %s in-pipe open\n",drvname[snum],0,0,0);
		return(-1);
	}
	PRINT("device [%s] in-pipe open [%s] %d\n",drvname[snum],s,pinfd[snum],0);
#endif
	if( info[snum] == NULL && (info[snum] = find_device_info(GDISP, drvname[snum], 1)) == NULL )
	{
		PRINT("error : find device [%s]\n",drvname[snum],0,0,0);
		return(-1);
	}
	PRINT("device [%s]\n",drvname[snum],0,0,0);
	if( dev[snum] == NULL && (dev[snum] = XOpenDevice(GDISP,info[snum]->id)) == NULL )
	{
		PRINT("error : open device [%s]\n",drvname[snum],0,0,0);
		return(-1);
	}
	PRINT("open device [%s]\n",drvname[snum],0,0,0);
#if 0
	ctype = DEVICE_RESOLUTION;
	if( dctl[snum] == NULL && (dctl[snum] = XGetDeviceControl(GDISP,dev[snum],ctype)) == NULL )
	{
		PRINT("error : get device [%s]\n",drvname[snum],0,0,0);
		return(-1);
	}
	PRINT("get device [%s]\n",drvname[snum],0,0,0);
#endif
	return(0);
}
int test_xidrv(int snum)
{
int res;
unsigned long a;
	if( dev[snum] == NULL )
		return(-1);
	a = 0xf8;
	res = XSetDeviceMode(GDISP,dev[snum],a);
	PRINT("test_xidrv [%s] res=%d \n",drvname[snum],res,a,0);
	return( 0 );
}
/*-------------------------------------------------------------
	raw-mode
*/
int set_raw_mode(int on)
{
int snum,cnt;
	for( cnt=snum=0 ; snum < MAX_SCREEN ; snum++ )
	{
		if( do_set_raw_mode(snum,on) != -1 )
			cnt++;
	}
	return( cnt ? 0 : -1 );
}
static int do_set_raw_mode(int snum,int on)
{
int res;
unsigned long a;
	if( dev[snum] == NULL )
		return(-1);
	a = on ? TS_Raw : TS_Scaled;
	res = XSetDeviceMode(GDISP,dev[snum],a);
	PRINT("set_raw_mode [%s] res=%d \n",drvname[snum],res,a,0);
	return( 0 );
}
/*-------------------------------------------------------------
	selector
*/
int is_selector_on()
{
int snum,cnt;
	for( cnt=snum=0 ; snum < MAX_SCREEN ; snum++ )
	{
		if( do_is_selector_on(snum) == 0 )
			cnt++;
	}
	return( cnt ? 0 : -1 );
}
static int do_is_selector_on(int snum)
{
int res;
unsigned long a;
#if USE_PIPE_COMM
u_short dat;
#endif
	if( dev[snum] == NULL )
		return(-1);
#if USE_PIPE_COMM
	res = send_dmcpkt(snum,GET_SELECTOR_CMD,(char *)&dat,sizeof(dat));
	//PRINT("is_selector_on [%s] res=%d \n",drvname[snum],res,0,0);
	if( res >= 0 && res <= 2 )
		return(res);
	return(-1);
#else
	a = GET_SELCT_MID_CMD;
	if( XSetDeviceMode(GDISP,dev[snum],a) == Success )
		return( 1 );
	a = GET_SELCT_RIGHT_CMD;
	if( XSetDeviceMode(GDISP,dev[snum],a) == Success )
		return( 2 );
	return( 0 );
#endif
}
/*-------------------------------------------------------------
	select-button
*/
int select_button(int bttn)	/* 0=left/1=middle/2=right */
{
int snum,cnt;
	for( cnt=snum=0 ; snum < MAX_SCREEN ; snum++ )
	{
		if( do_select_button(snum,bttn) != -1 )
			cnt++;
	}
	return( cnt ? 0 : -1 );
}
static int do_select_button(int snum,int bttn)	/* 0=left/1=middle/2=right */
{
int res;
unsigned long a;
#if USE_PIPE_COMM
u_short dat;
#endif
	if( dev[snum] == NULL )
		return(-1);
#if USE_PIPE_COMM
	dat = bttn;
	res = send_dmcpkt(snum,SET_SELECTOR_CMD,(char *)&dat,sizeof(dat));
	PRINT("select_tb [%s] res=%d \n",drvname[snum],res,0,0);
	return( res );
#else
	a = SELCT_CMD + bttn;
	res = XSetDeviceMode(GDISP,dev[snum],a);
	PRINT("select_tb [%s] res=%d cmd=$%x \n",drvname[snum],res,a,0);
	return( (res == 0 ? 0 : -1) );
#endif
}
/*-------------------------------------------------------------
	reset
*/
int reset_tb()
{
int snum,cnt;
	for( cnt=snum=0 ; snum < MAX_SCREEN ; snum++ )
	{
		if( do_reset_tb(snum) != -1 )
			cnt++;
	}
	return( cnt ? 0 : -1 );
}
static int do_reset_tb(int snum)
{
int res;
unsigned long a;
#if USE_PIPE_COMM
u_short dat;
#endif
	if( dev[snum] == NULL )
		return(-1);
#if USE_PIPE_COMM
	dat = 0;
	res = send_dmcpkt(snum,RESET_CMD,(char *)&dat,sizeof(dat));
	PRINT("reset_tb [%s] res=%d \n",drvname[snum],res,0,0);
	return(res);
#else
	a = RESET_CMD;
	res = XSetDeviceMode(GDISP,dev[snum],a);
	PRINT("reset_tb [%s] res=%d cmd=$%x \n",drvname[snum],res,a,0);
	return( (res == 0 ? 0 : -1) );
#endif
}
/*-------------------------------------------------------------
	id
*/
int set_id(int snum,int id)
{
	if( id < 0 || id > 1 )
		return(-1);
	return( set_mem(snum,2,(1+(id<<4))) );
}
int get_id(int snum)
{
int res;
	if( (res = get_mem(snum,2)) != -1 )
		return( (res>>4) & 0x03 );
	return(res);
}
/*-------------------------------------------------------------
	mem
*/
int set_mem(int snum,int addr,int dat)
{
#if USE_PIPE_COMM
int res;
dmc_pktrwmem p;
	if( dev[snum] == NULL )
		return(-1);
	p.addr = addr;
	p.dat  = dat;
	res = send_dmcpkt(snum,SET_MEM,(char *)&p.addr,sizeof(p)-sizeof(dmc_pktheader));
	PRINT("set_mem [%s] res=%d \n",drvname[snum],res,0,0);
	return(res);
#else
	return(-1);
#endif
}
int get_mem(int snum,int addr)
{
#if USE_PIPE_COMM
int res;
dmc_pktrwmem p;
	if( dev[snum] == NULL )
		return(-1);
	p.addr = addr;
	p.dat  = 0;
	res = send_dmcpkt(snum,GET_MEM,(char *)&p.addr,sizeof(p)-sizeof(dmc_pktheader));
	PRINT("get_mem [%s] res=%d($%x) \n",drvname[snum],res,res,0);
	return(res);
#else
	return(-1);
#endif
}
/*-------------------------------------------------------------
	samp-rate
*/
int set_samp_rate(int s_rate)
{
int snum,cnt;
	for( cnt=snum=0 ; snum < MAX_SCREEN ; snum++ )
	{
		if( do_set_samp_rate(snum,s_rate) != -1 )
			cnt++;
	}
	return( cnt ? 0 : -1 );
}
static int do_set_samp_rate(int snum,int s_rate)
{
int res;
unsigned long a;
#if USE_PIPE_COMM
u_short dat;
#endif
	if( dev[snum] == NULL )
		return(-1);
	if( s_rate < S_RATE_1st || s_rate > S_RATE_last )
		return(-1);
#if USE_PIPE_COMM
	dat = s_rate;
	res = send_dmcpkt(snum,SET_RATE_CMD,(char *)&dat,sizeof(dat));
	PRINT("set_samp_rate [%s] res=%d \n",drvname[snum],res,0,0);
	return(res);
#else
	a = S_RATE_CMD + s_rate;
	res = XSetDeviceMode(GDISP,dev[snum],a);
	PRINT("set_samp_rate [%s] res=%d cmd=$%x \n",drvname[snum],res,a,0);
	return( (res == 0 ? 0 : -1) );
#endif
}
int get_samp_rate()
{
int snum,res;
	for( res=snum=0 ; snum < MAX_SCREEN ; snum++ )
	{
		if( (res = do_get_samp_rate(snum)) != -1 )
			return(res);
	}
	return(-1);
}
static int do_get_samp_rate(int snum)
{
#if USE_PIPE_COMM
int res;
u_short dat;
	if( dev[snum] == NULL )
		return(-1);
	res = send_dmcpkt(snum,GET_RATE_CMD,(char *)&dat,sizeof(dat));
	PRINT("get samp_rate [%s] %d\n",drvname[snum],res,0,0);
	if( res >= S_RATE_1st && res <= S_RATE_last )
		return(res);
	return(-1);
#else
	return(0);
#endif
}
/*-------------------------------------------------------------
	scr_max
*/
int get_scr_max(int snum,int *x,int *y)
{
#if USE_PIPE_COMM
int res;
dmc_pktresinfo dat;
	if( dev[snum] == NULL )
		return(-1);
	memset(&dat,0,sizeof(dat));
	res = send_dmcinpkt(snum,GET_INFO_CMD,(char *)&dat,sizeof(dat));
	if( res == 0 )
	{
		*x = dat.max_x;
		*y = dat.max_y;
		PRINT(" screen num [%s] %d\n",drvname[snum],dat.screen_num,0,0);
		PRINT(" max_x [%s] %d\n",drvname[snum],dat.max_x,0,0);
		PRINT(" max_y [%s] %d\n",drvname[snum],dat.max_y,0,0);
		PRINT(" samp_rate [%s] %d\n",drvname[snum],dat.rate,0,0);
		PRINT(" beep [%s] %d\n",drvname[snum],dat.beep,0,0);
		PRINT(" select [%s] %d\n",drvname[snum],dat.select,0,0);
	}
	return(res);
#else
	return(0);
#endif
}
/*-------------------------------------------------------------
	calib-point
*/
int get_calib_point_num()
{
int snum,res;
	for( snum=0 ; snum < MAX_SCREEN ; snum++ )
	{
		if( (res=do_get_calib_point_num(snum)) != -1 )
			return(res);
	}
	return(-1);
}
static int do_get_calib_point_num(int snum)
{
#if USE_PIPE_COMM
int res;
u_short dat;
	if( dev[snum] == NULL )
		return(-1);
	res = send_dmcpkt(snum,GET_CALIBNUM_CMD,(char *)&dat,sizeof(dat));
	PRINT("get calib point num [%s] %d\n",drvname[snum],res,0,0);
	if( res >= 2 && res <= 9 )
		return(res);
	return(2);
#else
	return(2);
#endif
}
/*-------------------------------------------------------------
	beep
*/
int set_beep(int pen_down,int on)
{
int snum,cnt;
	for( cnt=snum=0 ; snum < MAX_SCREEN ; snum++ )
	{
		if( do_set_beep(snum,pen_down,on) != -1 )
			cnt++;
	}
	return( cnt ? 0 : -1 );
}
static int do_set_beep(int snum,int pen_down,int on)
{
int res;
unsigned long a;
#if USE_PIPE_COMM
u_short dat;
#endif
	if( dev[snum] == NULL )
		return(-1);
#if USE_PIPE_COMM
	if( pen_down )
		dat = on ? BEEP_DOWN_ON : BEEP_DOWN_OFF;
	else
		dat = on ? BEEP_UP_ON : BEEP_UP_OFF;
	res = send_dmcpkt(snum,SET_BEEP_CMD,(char *)&dat,sizeof(dat));
	PRINT("set_beep [%s] res=%d \n",drvname[snum],res,0,0);
	return(res);
#else
	if( pen_down )
		a = on ? BEEP_ON_DOWN_CMD : BEEP_OFF_DOWN_CMD;
	else
		a = on ? BEEP_ON_UP_CMD : BEEP_OFF_UP_CMD;
	res = XSetDeviceMode(GDISP,dev[snum],a);
	PRINT("set_beep [%s] res=%d cmd=$%x \n",drvname[snum],res,a,0);
	return( (res == 0 ? 0 : -1) );
#endif
}
int get_beep()
{
int snum,res;
	for( res=snum=0 ; snum < MAX_SCREEN ; snum++ )
	{
		if( (res=do_get_beep(snum)) != -1 )
			return(res);
	}
	return(-1);
}
static int do_get_beep(int snum)
{
#if USE_PIPE_COMM
int res;
u_short dat;
	if( dev[snum] == NULL )
		return(-1);
	res = send_dmcpkt(snum,GET_BEEP_CMD,(char *)&dat,sizeof(dat));
	PRINT("get beep [%s] %d\n",drvname[snum],res,0,0);
	if( res >= 0x00 && res <= 0x03 )
		return(res);
	return(-1);
#else
	return(0x03);
#endif
}
/*-------------------------------------------------------------
	lift-off-delay
*/
int set_offdelay(int msec)
{
int snum,cnt;
	for( cnt=snum=0 ; snum < MAX_SCREEN ; snum++ )
	{
		if( do_set_offdelay(snum,msec) != -1 )
			cnt++;
	}
	return( cnt ? 0 : -1 );
}
static int do_set_offdelay(int snum,int msec)
{
#if USE_PIPE_COMM
int res;
u_short dat;
	if( dev[snum] == NULL )
		return(-1);
	dat = (u_short)msec;
	res = send_dmcpkt(snum,SET_OFFDELAY,(char *)&dat,sizeof(dat));
	PRINT("set_offdelay [%s] res=%d %dmsec\n",drvname[snum],res,msec,0);
	return(res);
#else
	return( 0 );
#endif
}
int get_offdelay()
{
int snum,res;
	for( res=snum=0 ; snum < MAX_SCREEN ; snum++ )
	{
		if( (res=do_get_offdelay(snum)) != -1 )
			return(res);
	}
	return(-1);
}
static int do_get_offdelay(int snum)
{
#if USE_PIPE_COMM
int res;
u_short dat;
	if( dev[snum] == NULL )
		return(-1);
	res = send_dmcpkt(snum,GET_OFFDELAY,(char *)&dat,sizeof(dat));
	PRINT("get offdelay [%s] %d\n",drvname[snum],res,0,0);
	return(res);
#else
	return(0);
#endif
}
/*-------------------------------------------------------------
	save
*/
int save_exec()
{
int snum,cnt;
	for( cnt=snum=0 ; snum < MAX_SCREEN ; snum++ )
	{
		if( do_save_exec(snum) != -1 )
			cnt++;
	}
	return( cnt ? 0 : -1 );
}
static int do_save_exec(int snum)
{
#if USE_PIPE_COMM
int res;
u_short dat;
	if( dev[snum] == NULL )
		return(-1);
	res = send_dmcpkt(snum,SAVE_CMD,(char *)&dat,sizeof(dat));
	PRINT("set_save [%s] res=%d \n",drvname[snum],res,0,0);
	return(res);
#else
	return(0);
#endif
}
int set_next_calib(int snum)
{
int res;
unsigned long a;
#if USE_PIPE_COMM
u_short dat;
#endif
	if( dev[snum] == NULL )
		return(-1);
#if USE_PIPE_COMM
	dat = 0;
	res = send_dmcpkt(snum,CALIB_NEXT_CMD,(char *)&dat,sizeof(dat));
	PRINT("set_next_calib [%s] res=%d \n",drvname[snum],res,0,0);
	return(res);
#else
	a = CALIB_NEXT_CMD;
	res = XSetDeviceMode(GDISP,dev[snum],a);
	PRINT("set_next_calib [%s] res=%d cmd=$%x \n",drvname[snum],res,a,0);
	return( (res == 0 ? 0 : -1) );
#endif
}
int set_end_calib(int snum)
{
int res;
unsigned long a;
#if USE_PIPE_COMM
u_short dat;
#endif
	if( dev[snum] == NULL )
		return(-1);
#if USE_PIPE_COMM
	dat = 0;
	res = send_dmcpkt(snum,CALIB_END_CMD,(char *)&dat,sizeof(dat));
	PRINT("set_end_calib [%s] res=%d \n",drvname[snum],res,0,0);
	return(res);
#else
	a = CALIB_END_CMD;
	res = XSetDeviceMode(GDISP,dev[snum],a);
	PRINT("set_end_calib [%s] res=%d cmd=$%x \n",drvname[snum],res,a,0);
	return( (res == 0 ? 0 : -1) );
#endif
}
int cancel_calib(int snum)
{
int res;
unsigned long a;
#if USE_PIPE_COMM
u_short dat;
#endif
	if( dev[snum] == NULL )
		return(-1);
#if USE_PIPE_COMM
	dat = 0;
	res = send_dmcpkt(snum,CALIB_CANCEL_CMD,(char *)&dat,sizeof(dat));
	PRINT("cancel_calib [%s] res=%d \n",drvname[snum],res,0,0);
	return(res);
#else
	a = CALIB_CANCEL_CMD;
	res = XSetDeviceMode(GDISP,dev[snum],a);
	PRINT("cancel_calib [%s] res=%d cmd=$%x \n",drvname[snum],res,a,0);
	return( (res == 0 ? 0 : -1) );
#endif
}
int ok_calib(int snum)
{
int res;
unsigned long a;
#if USE_PIPE_COMM
u_short dat;
#endif
	if( dev[snum] == NULL )
		return(-1);
#if USE_PIPE_COMM
	dat = 0;
	res = send_dmcpkt(snum,CALIB_OK_CMD,(char *)&dat,sizeof(dat));
	PRINT("ok_calib [%s] res=%d \n",drvname[snum],res,0,0);
	return(res);
#else
	a = CALIB_OK_CMD;
	res = XSetDeviceMode(GDISP,dev[snum],a);
	PRINT("ok_calib [%s] res=%d cmd=$%x \n",drvname[snum],res,a,0);
	return( (res == 0 ? 0 : -1) );
#endif
}
int set_start_calib(int snum,uRect adj[],int count)
{
int res,i,n;
unsigned long a;
#if USE_PIPE_COMM
dmc_pktcalib pkt;
#endif
	PRINT("set_start_calib : XSetDeviceMode -> SwitchMode %d %d\n",snum,count,0,0);
	if( dev[snum] == NULL )
		return(-1);
	if( count > 9 || count < 2 )
		return(-1);
#if USE_PIPE_COMM
	pkt.calib_point = count;
	memcpy(&pkt.adj,adj,sizeof(pkt.adj));
	res = send_dmcpkt(snum,CALIB_FIRST_CMD,(char *)&pkt.calib_point
					,sizeof(dmc_pktcalib)-sizeof(dmc_pktheader));
	PRINT("set_start_calib [%s] res=%d cmd=$%x \n",drvname[snum],res,a,0);
	return(res);
#else
	a = CALIB_PRE_FIRST_CMD;
	res = XSetDeviceMode(GDISP,dev[snum],a);
	PRINT("set_start_calib [%s] res=%d cmd=$%x \n",drvname[snum],res,a,0);
	for( i=0 ; i < count ; i++ )
	{
		a = CALIB_ADJ_DAT_CMD + i;
		res = XSetDeviceMode(GDISP,dev[snum],a);
		PRINT("set_start_calib [%s] res=%d cmd=$%x \n",drvname[snum],res,a,0);
		do_set_calib(snum,0);	// raw x
		do_set_calib(snum,0);	// raw y
		do_set_calib(snum,(u_short)adj[i].x);
		do_set_calib(snum,(u_short)adj[i].y);
	}
	a = CALIB_FIRST_CMD;
	res = XSetDeviceMode(GDISP,dev[snum],a);
	PRINT("set_start_calib [%s] res=%d cmd=$%x \n",drvname[snum],res,a,0);
	return( (res == 0 ? 0 : -1) );
#endif
}
#if !defined(USE_PIPE_COMM)
static int do_set_calib(int snum,u_short dat)
{
int n,res;
u_char a;
	for( n=12 ; n >= 0 ; n-=4 )
	{
		a = CALIB_1_DAT | ((dat >> n) & 0x0f);
		res = XSetDeviceMode(GDISP,dev[snum],a);
		//PRINT("set mode [%s] res=%d dat=$%x \n",drvname[snum],res,a,0);
	}
	return(0);
}
#endif
static XDeviceInfo *find_device_info(Display *display,char *name, Bool only_extended)
{
XDeviceInfo *devices;
int     loop;
int     num_devices;
int     len = strlen(name);
Bool    is_id = True;
XID     id;
    for( loop=0 ; loop < len; loop++ ) 
	{
	    if( !isdigit(name[loop]) ) 
		{
	        is_id = False;
	        break;
	    }
	}
	if( is_id ) 
	    id = atoi(name);
    devices = XListInputDevices(display, &num_devices);
#if 1
    for( loop=0 ; loop < num_devices ; loop++ ) 
	{
	#if defined(XVERS14)
		if( devices[loop].use == IsXExtensionDevice 
		 || devices[loop].use == IsXExtensionPointer )
	#else
		if( devices[loop].use == IsXExtensionDevice  )
	#endif
		{
			printf("DEVICE ID=$%04x use=$%04x %s\n"
				,devices[loop].id
				,devices[loop].use
				,devices[loop].name
			);
		}
	}
#endif
    for( loop=0 ; loop < num_devices ; loop++ ) 
	{
	#if defined(XVERS14)
	    if( (!only_extended || (devices[loop].use == IsXExtensionDevice)
	                        || (devices[loop].use == IsXExtensionPointer))
	#else
	    if( (!only_extended || (devices[loop].use == IsXExtensionDevice))
	#endif
		 && ((!is_id && strcmp(devices[loop].name, name) == 0)
								|| (is_id && devices[loop].id == id)))
		{
	        return &devices[loop];
	    }
    }
    return NULL;
}
#if USE_PIPE_COMM

static int pipe_open(char *name)
{
int fd,flg;
	flg = O_RDWR;
	if( (fd = open(name,flg)) == -1 )
	{
		mkfifo(name,0666);
		fd = open(name,flg);
	}
	return(fd);
}
static int send_dmcpkt(int snum,int cmd,char *dat,int size)
{
int res;
dmc_pkt pkt;
	pkt.u.pnorm.hd.cmd = cmd;
	pkt.u.pnorm.hd.rsv = 0xa5a5;
	switch( pkt.u.pnorm.hd.cmd )
	{
	case	CALIB_FIRST_CMD:
		memcpy(&pkt.u.pcalib.calib_point,dat,size);
		break;
	case	SET_MEM:
	case	GET_MEM:
		memcpy(&pkt.u.rwmem.addr,dat,size);
		break;
	default:
		memcpy(&pkt.u.pnorm.dat,dat,size);
		break;
	}
	write(poutfd[snum],&pkt,sizeof(dmc_pktheader)+size);
//printf("DEBUG:write pout(%d) cmd=%d siz=%d %d $%08x $%08x\n",poutfd[snum],pkt.u.pnorm.hd.cmd,size,snum,gDisp,GDISP);
	if( (res = XSetDeviceMode(GDISP,dev[snum],XDEV_KICK)) == 0 )
	{
		res = recv_dmcpkt(snum,cmd);
	}
	else
		res = -1;
	return(res);
}
static int recv_dmcpkt(int snum,int cmd)
{
dmc_pktres pres;
	if( read(pinfd[snum],&pres,sizeof(dmc_pktres)) != sizeof(dmc_pktres) )
		return(-1);
//printf("DEBUG:read pin(%d) cmd=%d:%d res=%d\n",pinfd[snum],pres.hd.cmd,cmd,pres.res);
	if( pres.hd.cmd == cmd )
		return( pres.res );
	return(-1);
}
static int send_dmcinpkt(int snum,int cmd,char *dat,int size)
{
int sz,res;
dmc_pkt pkt;
	sz = sizeof(dmc_pktnorm);
	pkt.u.pnorm.hd.cmd = cmd;
	pkt.u.pnorm.hd.rsv = 0xa5a5;
	write(poutfd[snum],&pkt,sz);
	if( (res = XSetDeviceMode(GDISP,dev[snum],XDEV_KICK)) == 0 )
	{
		res = recv_dmcinpkt(snum,cmd,dat,size);
	}
	else
		res = -1;
	return(res);
}
static int recv_dmcinpkt(int snum,int cmd,void *dat,int size)
{
dmc_pkt *p;
printf("read_dmcinpkt %d %d\n",snum,size);
	p = (dmc_pkt *)dat;
	if( read(pinfd[snum],dat,size) != size )
		return(-1);
	if( p->u.pres.hd.cmd == cmd )
		return( p->u.pres.res );
	return(-1);
}
#endif

