/*
 * Copyright (c) 2004  DMC Co., Ltd.
 *
 * Copyright (c) 1999  Machine Vision Holdings Incorporated
 * Author: Mayk Langer <langer@vsys.de>
 *
 * Template driver used: Copyright (c) 1998  Metro Link Incorporated
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE X CONSORTIUM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF
 * OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 * Except as contained in this notice, the name of the Metro Link shall not be
 * used in advertising or otherwise to promote the sale, use or other dealings
 * in this Software without prior written authorization from Metro Link.
 *
 */
/* $XFree86: xc/programs/Xserver/hw/xfree86/input/dmc/xf86DMC.h,v 1.1 2001/08/17 13:27:55 dawes Exp $ */

#ifndef	_DMC_H_
#define _DMC_H_

/******************************************************************************
 *		Definitions
 *				structs, typedefs, #defines, enums
 *****************************************************************************/
#define DMC_PACKET_SIZE		5

typedef enum
{
	DMC_byte0, DMC_byte1, DMC_byte2, DMC_byte3, DMC_byte4,
	DMC_Response0,DMC_Response1
}
DMCState;

typedef struct uPoint
{
	int x,y;
} uPoint;

typedef struct uARect
{
	uPoint lefttop;
	uPoint leftbottom;
	uPoint righttop;
	uPoint rightbottom;
} uARect;

#include <linux/types.h>
//typedef int64_t NTIME;
typedef __s64 NTIME;

typedef struct DMCSaveFile
{
	int point;
	int tp_state;
	int samp_rate;
	int offdelay;
	unsigned int raw[16][2];
	unsigned int adj[16][2];
} DMCSaveFile;

typedef struct _DMCPrivateRec
{
	InputInfoPtr pInfo;
	int usb_dev;
	int fd;
	int min_x;				/* Minimum x reported by calibration        */
	int max_x;				/* Maximum x                    */
	int min_y;				/* Minimum y reported by calibration        */
	int max_y;				/* Maximum y                    */
	Bool button_down;			/* is the "button" currently down */
	int button_number;			/* which button to report */
	int reporting_mode;			/* TS_Raw or TS_Scaled */
	NTIME reporting_time;			

	int screen_num;				/* Screen associated with the device        */
	int screen_width;			/* Width of the associated X screen     */
	int screen_height;			/* Height of the screen             */
	int proximity;
	int swap_xy;
	int reverse_xy;
	int init_beep_on;

	DMCSaveFile file_data;
	int tp_state;
#define	B_BELL_DOWN		1
#define	B_BELL_UP		2
#define	B_SELECT_LEFT	0x10
#define	B_SELECT_MID	0x20
#define	B_SELECT_RIGHT	0x40
#define	MASK_SELECT		0x70
	int samp_rate;
#define	START_S_RATE	S_RATE_1st
#define	END_S_RATE		S_RATE_last
#define	RATE_PMODE		S_RATE_last
	int offdelay;
	XISBuffer *buffer;
	unsigned char packet[DMC_PACKET_SIZE];	/* packet being/just read */
	DMCState lex_mode;
	char pen_down;
	/* ...................calibration data */
	int eeprom_read_ok;
	int calib_raw_x;
	int calib_raw_y;
	int do_calib;
	/**/
	int calib_point;
#define	MAX_CALIB_POINT	9
#define	MAX_DATA		4
	unsigned int calib_data[16][MAX_DATA];
	uARect calib_run_raw_r[4];
	uARect calib_run_adj_r[4];
	/* ...................calibration temopary data */
	#if !defined(USE_PIPE_COMM)
		int calib_temp_nibble;
	#endif
	unsigned int calib_temp_raw_data[16][2];
	/**/
	int calib_temp_point;
	unsigned int calib_temp_data[16][MAX_DATA];
	uARect calib_temp_run_raw_r[4];
	uARect calib_temp_run_adj_r[4];
	/**/
	long test;
	char *save_file;
	#if defined(USE_PIPE_COMM)
		char *apidir;
		int pin;
		int pout;
	#endif
	//#if XORG_NEWVERS14
		int consfd;
	//#endif
	char *modpath;
}
DMCPrivateRec, *DMCPrivatePtr;

typedef struct Pkt_Calib 
{
	unsigned short rawx,rawy;
	unsigned short refx,refy;
} Pkt_Calib;

typedef struct Pkt_W_EEPROM 
{
	unsigned char x;
	unsigned char y;
	Pkt_Calib x00;	// +---------------+
	Pkt_Calib x01;	// | x00  x01  x02 |
	Pkt_Calib x02;	// | x10  x11  x12 |
	Pkt_Calib x10;	// | x20  x21  x22 |
	Pkt_Calib x11;	// +---------------+
	Pkt_Calib x12;
	Pkt_Calib x20;
	Pkt_Calib x21;
	Pkt_Calib x22;
	unsigned short max_x;
	unsigned short max_y;
} Pkt_W_EEPROM;
 
typedef struct Pkt_R_EEPROM 
{
	unsigned char x;
	unsigned char y;
	Pkt_Calib x00;	// +---------------+---------+
	Pkt_Calib x01;	// | x00  x01  x02 | x03 x04 |
	Pkt_Calib x02;	// | x10  x11  x12 | x13 x14 |
	Pkt_Calib x03;	// | x20  x21  x22 | x23 x24 |
	Pkt_Calib x04;	// +---------------+---------+
	Pkt_Calib x10;	// | x30  x31  x32 | x33 x34 |
	Pkt_Calib x11;	// | x40  x41  x42 | x43 x44 |
	Pkt_Calib x12;	// +---------------+---------+
	Pkt_Calib x13;
	Pkt_Calib x14;
	Pkt_Calib x20;
	Pkt_Calib x21;
	Pkt_Calib x22;
	Pkt_Calib x23;
	Pkt_Calib x24;
	Pkt_Calib x30;
	Pkt_Calib x31;
	Pkt_Calib x32;
	Pkt_Calib x33;
	Pkt_Calib x34;
	Pkt_Calib x40;
	Pkt_Calib x41;
	Pkt_Calib x42;
	Pkt_Calib x43;
	Pkt_Calib x44;
	unsigned short max_x;
	unsigned short max_y;
} Pkt_R_EEPROM;
 
/******************************************************************************
 *		Declarations
 *****************************************************************************/
/*
 * tbcalib's calib point mark size
 */
#define	ADJ_X	15
#define	ADJ_Y	15
#define	W_MARK	20
#define	H_MARK	20

static Bool DeviceControl (DeviceIntPtr, int);
static void ReadInput (InputInfoPtr);
static int ControlProc (InputInfoPtr, xDeviceCtl *);
static void CloseProc (InputInfoPtr);
static int SwitchMode (ClientPtr, DeviceIntPtr, int);
static Bool ConvertProc (InputInfoPtr, int, int, int, int, int, int, int, int, int *, int *);
static Bool QueryHardware (DMCPrivatePtr);
static Bool DMCGetPacket (DMCPrivatePtr priv);
static Bool DMCSendPacket (DMCPrivatePtr priv, unsigned char *buf, int len );
#if XORG_NEWVERS110
static int DMCPreInit(InputDriverPtr drv, InputInfoPtr pInfo, int flags);
#else
static InputInfoPtr DMCPreInit(InputDriverPtr drv, IDevPtr dev, int flags);
#endif
static void DMCPtrCtrl(DeviceIntPtr device, PtrCtrl *ctrl);

#endif /* _DMC_H_ */
